unit BRTF_Shared;

interface

uses
  comctrls,graphics;

procedure ChangeSelectionStyles(RichEdit: TCustomRichEdit; FontStyles: TFontStyles; Add: boolean);
procedure EditDBRichEdit(RichEdit: TCustomRichEdit);
function FontStyleToConAttr(FS: TFontStyle): TConsistentAttribute;



implementation

uses
  Classes,RichEdit,dbctrls,Windows,db,sysutils;

// Primenjuje font stilove na selektovani tekst. Ako je Add=true, dodaje
// stilove, a u suprotnom ih uklanja. Ako je portebno, poziva EditDBRichEdit.
procedure ChangeSelectionStyles(RichEdit : TCustomRichEdit; FontStyles : TFontStyles; Add : boolean);
var
  OldChange,OldSelChange : TNotifyEvent;
  OldSelStart,OldSelLength,i : integer;
  Format: TCharFormat;
begin

  // Jer je TCustomRichEdit.OnSelectionChange protected.
  if (RichEdit is TRichEdit) then
    with TRichEdit(RichEdit) do
    begin
      OldChange := OnChange;
      OnChange := nil;
      OldSelChange := OnSelectionChange;
      OnSelectionChange := nil
    end
  else if (RichEdit is TDBRichEdit) then
    with TDBRichEdit(RichEdit) do
    begin
      OldChange := OnChange;
      OnChange := nil;
      OldSelChange := OnSelectionChange;
      OnSelectionChange := nil;
      EditDBRichEdit(RichEdit)
    end
  else
  begin
    OldChange := nil;
    OldSelChange := nil
  end;

  //OldSelChange := RichEdit.OnSelectionChange;
  //RichEdit.OnSelectionChange := nil;
  OldSelStart := RichEdit.SelStart;
  OldSelLength := RichEdit.SelLength;

  if OldSelLength=0 then
  begin

    //InitFormat(Format);
    FillChar(Format, SizeOf(TCharFormat), 0);
    Format.cbSize := SizeOf(TCharFormat);

    with Format do
    begin

      if fsBold in FontStyles then
        dwMask := dwMask or CFM_BOLD
      else if fsItalic in FontStyles then
        dwMask := dwMask or CFM_ITALIC
      else if fsUnderline in FontStyles then
        dwMask := dwMask or CFM_UNDERLINE
      else if fsStrikeout in FontStyles then
        dwMask := dwMask or CFM_STRIKEOUT;

      //dwMask := CFM_BOLD or CFM_ITALIC or CFM_UNDERLINE or CFM_STRIKEOUT;

      if Add and (fsBold in FontStyles) then dwEffects := dwEffects or CFE_BOLD;
      if Add and (fsItalic in FontStyles) then dwEffects := dwEffects or CFE_ITALIC;
      if Add and (fsUnderline in FontStyles) then dwEffects := dwEffects or CFE_UNDERLINE;
      if Add and (fsStrikeOut in FontStyles) then dwEffects := dwEffects or CFE_STRIKEOUT;

    end;

    if RichEdit.HandleAllocated then
      SendMessage(RichEdit.Handle, EM_SETCHARFORMAT, SCF_SELECTION, LPARAM(@Format))

    (*
    with RichEdit.DefAttributes do
    begin
      if Add then
        Style := Style + FontStyles
      else
        Style := Style - FontStyles
    end
    *)
  end
  else
  begin
    RichEdit.Lines.BeginUpdate;
    try

      with RichEdit do  // Radi ubrzanja.
      begin
        for i := OldSelStart to OldSelStart+OldSelLength-1 do
        begin
          SelStart := i;
          SelLength := 1;
          with SelAttributes do  // Radi ubrzanja.
            if Add then
              Style := Style + FontStyles
            else
              Style := Style - FontStyles
        end
      end;

    finally
      RichEdit.Lines.EndUpdate
    end;
  end;

  RichEdit.SelStart := OldSelStart;
  RichEdit.SelLength := OldSelLength;

  if (RichEdit is TRichEdit) then
    with TRichEdit(RichEdit) do
    begin
      OnChange := OldChange;
      OnSelectionChange := OldSelChange;
      //if Assigned(OnChange) then
        //OnChange(RichEdit)
    end
  else if (RichEdit is TDBRichEdit) then
    with TDBRichEdit(RichEdit) do
    begin
      OnChange := OldChange;
      OnSelectionChange := OldSelChange;
      //if Assigned(OnChange) then
        //OnChange(RichEdit)
    end;

  //RichEdit.OnSelectionChange := OldSelChange;
end;

// Ako je RichEdit vezan za polje u bazi, stavlja njegov
// DataSet u Edit stranje uz ocuvanje selekcije u RichEdit-u.
procedure EditDBRichEdit(RichEdit: TCustomRichEdit);
var
  OldSelStart, OldSelLength : integer;
  OldChange, OldSelectionChange : TNotifyEvent;
  DBRichEdit : TDBRichEdit;
begin
  // Stavljamo DBRichEdit.DataSource u Edit stanje.
  if (RichEdit is TDBRichEdit) then
  begin
    DBRichEdit := RichEdit as TDBRichEdit;
    if Assigned(DBRichEdit.DataSource) and not (DBRichEdit.DataSource.State in [dsInsert,dsEdit]) then
    begin
      // Cuvamo se od nezeljenih poziva OnSelectionChange.
      OldSelectionChange := DBRichEdit.OnSelectionChange;
      DBRichEdit.OnSelectionChange := nil;
      OldChange := DBRichEdit.OnChange;
      DBRichEdit.OnChange := nil;
      OldSelStart := DBRichEdit.SelStart;
      OldSelLength := DBRichEdit.SelLength;
      try
        DBRichEdit.DataSource.DataSet.Edit
      finally
        DBRichEdit.SelStart := OldSelStart;
        DBRichEdit.SelLength := OldSelLength;
        DBRichEdit.OnSelectionChange := OldSelectionChange;
        DBRichEdit.OnChange := OldChange
      end
    end;
  end; // (FRichEdit is TDBRichEdit)
end;

function FontStyleToConAttr(FS: TFontStyle): TConsistentAttribute;
begin
  case FS of
    fsBold:
      result := caBold;
    fsItalic:
      result := caItalic;
    fsUnderline:
      result := caUnderline;
    fsStrikeout:
      result := caStrikeout;
  else
    raise Exception.Create('FontStyleToConsistentAttribute() Invalid style.')
  end

end;



end.
