unit FormatButtons;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, comctrls;

type

  // Bazna klasa za sve format dugmice.
  TCustomRichButton = class(TSpeedButton)
  private
    FRichEdit: TCustomRichEdit;
    FGrayed: boolean;
    Pattern: TBitmap;

    procedure Init;
    procedure SetRichEdit(const Value: TCustomRichEdit);
    //procedure SetGrayed(const Value: boolean);
    procedure ReadRichState;

  protected

    Bitmap: TBitmap;
    //procedure RefreshRichTextButton; virtual; abstract;
    procedure SetParent(AParent: TWinControl); override;
    constructor CreateFromBitMapRes(AOwner: TComponent; const ResName: string);
    procedure Paint; override;

    // Preko ovih funkcija njena deca komniciraju sa ovom klasom.
    function FontStyle(var FS: TFontStyle): boolean; virtual;
    function ParagraphAlignment(var PA: TAlignment): boolean; virtual;
    function Numbering(var NS: TNumberingStyle): boolean; virtual;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure Refresh;
    property Grayed: boolean read FGrayed; // write SetGrayed;

  published
    property RichEdit: TCustomRichEdit read FRichEdit write SetRichEdit;

  end;

  // FontStyle dugmad.

  TBoldButton = class(TCustomRichButton)
  protected
    function FontStyle(var FS: TFontStyle): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TItalicButton = class(TCustomRichButton)
  protected
    function FontStyle(var FS: TFontStyle): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TUnderlineButton = class(TCustomRichButton)
  protected
    function FontStyle(var FS: TFontStyle): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TStrikeoutButton = class(TCustomRichButton)
  protected
    function FontStyle(var FS: TFontStyle): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  // ParagraphAlignment dugmad.

  TAlignLeftButton = class(TCustomRichButton)
  protected
    function ParagraphAlignment(var PA: TAlignment): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TAlignCenterButton = class(TCustomRichButton)
  protected
    function ParagraphAlignment(var PA: TAlignment): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TAlignRightButton = class(TCustomRichButton)
  protected
    function ParagraphAlignment(var PA: TAlignment): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  // Numbering dugme.

  TBulletButton = class(TCustomRichButton)
  protected
    function Numbering(var NS: TNumberingStyle): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

procedure Register;
procedure RefreshRichControls(Parent: TWinControl);



implementation

uses
  BRTF_Shared,Math,dbctrls;

{$R *.DCR}

var
  InClick: boolean; // Synchronization with Refresh.

(*
  LoadBitmap('OPEN');
  LoadBitmap('SAVE');

  LoadBitmap('CUT');
  LoadBitmap('COPY');
  LoadBitmap('PASTE');

  LoadBitmap('FONT');
  LoadBitmap('CODEPAGE');
  LoadBitmap('COLOR');

  LoadBitmap('BOLD');
  LoadBitmap('ITALIC');
  LoadBitmap('UNDERLINE');
  LoadBitmap('STRIKEOUT');
  LoadBitmap('ALIGNLEFT');
  LoadBitmap('ALIGNCENTER');
  LoadBitmap('ALIGNRIGHT');

  LoadBitmap('CLEARFORMATTING');
  LoadBitmap('INSERTIMAGE');
*)


procedure Register;
begin
  RegisterComponents('BRTF',
    [TBoldButton,TItalicButton,TUnderlineButton,TStrikeoutButton,
    TAlignLeftButton,TAlignCenterButton,TAlignRightButton,
    TBulletButton]);
end;

procedure RefreshRichControls(Parent: TWinControl);
var
  i: integer;
begin
  if not Assigned(Parent) then
    exit;
  with Parent do
    for i := 0 to ControlCount-1 do
      if Controls[i] is TCustomRichButton then
        TCustomRichButton(Controls[i]).Refresh
end;



{ TCustomRichButton }

////////////////////////////////////////////////////////////////////////////////
// Private.

// Zajednicki kod za konstruktore.
procedure TCustomRichButton.Init;
begin
  Bitmap := TBitmap.Create;
  // PatternManager ce se pobrinuti da se ova bitmapa oslobodi,
  // pa nam ne treba taj kod u destruktoru.
  Pattern := AllocPatternBitmap(clBtnFace, clBtnHighlight);
  Transparent := false;
end;

procedure TCustomRichButton.SetRichEdit(const Value: TCustomRichEdit);
begin
  FRichEdit := Value;
  Refresh
end;

(*
procedure TCustomRichButton.SetGrayed(const Value: boolean);
begin
  FGrayed := Value;
  Paint
end;
*)

// Podesava Grayed i Down na osnovu formatiranja selekcije RichEdit-a.
procedure TCustomRichButton.ReadRichState;
var
  FS: TFontStyle;
  PA: TAlignment;
  NS: TNumberingStyle;
begin
  // Bilo bi dobro kada bi se moglo proveriti i da li je pozvan
  // RichEdit.Lines.BeginUpdate.
  if Assigned(RichEdit) then
  begin
    if FontStyle(FS) then
    begin
      if RichEdit.SelLength > 0 then
        FGrayed := not (FontStyleToConAttr(FS) in RichEdit.SelAttributes.ConsistentAttributes)
      else
        FGrayed := false;  
      if not FGrayed then
        Down := FS in RichEdit.SelAttributes.Style;
    end
    else if ParagraphAlignment(PA) then
      Down := PA = RichEdit.Paragraph.Alignment
    else if Numbering(NS) then
      Down := NS = RichEdit.Paragraph.Numbering
  end;
end;



////////////////////////////////////////////////////////////////////////////////
// Protected.

// Menjamo GroupIndex tako da omogucmo "toggle" efekat.
procedure TCustomRichButton.SetParent(AParent: TWinControl);
var
  i,MaxGroupIndex: integer;
begin
  inherited SetParent(AParent);
  if (GroupIndex=0) and Assigned(Parent) then
  begin
    // Trazimo jedinstveni GroupIndex.
    MaxGroupIndex := 0;
    for i := 0 to Parent.ControlCount-1 do
      if (Parent.Controls[i] is TSpeedButton) then
        MaxGroupIndex := Max(MaxGroupIndex,(Parent.Controls[i] as TSpeedButton).GroupIndex);
    // Dodeljujemo ga sebi.
    GroupIndex := MaxGroupIndex+1;
    AllowAllUp := True
  end
end;

// Koriste ga nasledjene kompoonente da bi odmah dodelili odgovarajucu
// bitmapu Glyph-u.
constructor TCustomRichButton.CreateFromBitMapRes(AOwner: TComponent;
  const ResName: string);
begin
  inherited Create(AOwner);
  Init;
  Bitmap.LoadFromResourceName(HInstance,ResName);
  Glyph := Bitmap
end;

procedure TCustomRichButton.Paint;
var
//  PaintRect: TRect;
//  OldTransparent: boolean;
//  OldColor: TColor;
//  Bmp: TBitmap;
  OldButtonState: TButtonState;
begin

  if not Assigned(Parent) then
    raise Exception.Create('TCustomRichButton.Paint() Parent is not assigned.');

  OldButtonState := FState;
  try
    if not Grayed then
    begin
      if (FState = bsExclusive) then
        FState := bsDown
    end
    else
      FState := bsExclusive;
    inherited Paint;
  finally
    FState := OldButtonState;
  end;

  (*
  if Transparent then
  begin
    if Grayed then
      Canvas.Brush.Color := clInfoBk
    else
      Canvas.Brush.Color := Color;
    Canvas.FrameRect(Rect(2,2,Width-3,Height-3))
  end
  else if Grayed then
  begin
    Canvas.Brush.Bitmap := Pattern;
    Canvas.FloodFill(2,2,Canvas.Pixels[2,2],fsSurface)
  end
  *)

end;

// Vraca true ako je dugme zaduzeno za stil, u kom slucaju u
// 'FS' treba staviti stil za koji je dugme zaduzeno.
function TCustomRichButton.FontStyle(var FS: TFontStyle): boolean;
begin
  result := false
end;

function TCustomRichButton.ParagraphAlignment(var PA: TAlignment): boolean;
begin
  result := false
end;

function TCustomRichButton.Numbering(var NS: TNumberingStyle): boolean;
begin
  result := false
end;

procedure TCustomRichButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = RichEdit) then
    RichEdit := nil;
  inherited  
end;



////////////////////////////////////////////////////////////////////////////////
// Public.

constructor TCustomRichButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Init;
  InClick := false;
end;

destructor TCustomRichButton.Destroy;
begin
  Bitmap.Free;
  inherited Destroy
end;

procedure TCustomRichButton.Click;
var
  FS: TFontStyle;
  PA: TAlignment;
  NS: TNumberingStyle;
begin
  inherited Click;

  if InClick then
    exit;

  InClick := true;

  try

    if FGrayed then
    begin
      FGrayed := false;
      Down := false;
      Paint
    end;
    if Assigned(RichEdit) then
    begin
      EditDBRichEdit(RichEdit);

      if FontStyle(FS) then
        ChangeSelectionStyles(RichEdit, [FS], Self.Down)
      else if ParagraphAlignment(PA) then
      begin
        RichEdit.Paragraph.Alignment := PA
      end
      else if Numbering(NS) then
      begin
        if Down then
          RichEdit.Paragraph.Numbering := NS
        else
          RichEdit.Paragraph.Numbering := nsNone
      end
    end

  finally
    InClick := false
  end

end;

procedure TCustomRichButton.Refresh;
//var
//  OldSelectionChange, OldChange: TNotifyEvent;
begin

  if InClick then
    exit;

  (*
  if
    Assigned(RichEdit)
    and (RichEdit is TDBRichEdit)
    and Assigned(TDBRichEdit(RichEdit).Field)
    and TDBRichEdit(RichEdit).Field.IsNull
  then
  with TDBRichEdit(RichEdit) do
  begin
    OldSelectionChange := OnSelectionChange;
    OnSelectionChange := nil;
    OldChange := OnChange;
    OnChange := nil;

    SelAttributes.Style := [];
    Paragraph.Alignment := taLeftJustify;
    Paragraph.Numbering := nsNone;

    OnChange := OldChange;
    OnSelectionChange := OldSelectionChange
  end;
  *)

  ReadRichState;
  inherited Refresh
end;



{ TBoldButton }

constructor TBoldButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'BOLD')
end;

function TBoldButton.FontStyle(var FS: TFontStyle): boolean;
begin
  FS := fsBold;
  result := true
end;

{ TItalicButton }

constructor TItalicButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'ITALIC')
end;

function TItalicButton.FontStyle(var FS: TFontStyle): boolean;
begin
  FS := fsItalic;
  result := true
end;

{ TUnderlineButton }

constructor TUnderlineButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'UNDERLINE')
end;

function TUnderlineButton.FontStyle(var FS: TFontStyle): boolean;
begin
  FS := fsUnderline;
  result := true;
end;

{ TStrikeoutButton }

constructor TStrikeoutButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'STRIKEOUT')
end;

function TStrikeoutButton.FontStyle(var FS: TFontStyle): boolean;
begin
  FS := fsStrikeout;
  result := true
end;

{ TAlignLeftButton }

constructor TAlignLeftButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'ALIGNLEFT')
end;

function TAlignLeftButton.ParagraphAlignment(var PA: TAlignment): boolean;
begin
  PA := taLeftJustify;
  result := true
end;

{ TAlignCenterButton }

constructor TAlignCenterButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'ALIGNCENTER')
end;

function TAlignCenterButton.ParagraphAlignment(
  var PA: TAlignment): boolean;
begin
  PA := taCenter;
  result := true
end;

{ TAlignRightButton }

constructor TAlignRightButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'ALIGNRIGHT')
end;

function TAlignRightButton.ParagraphAlignment(var PA: TAlignment): boolean;
begin
  PA := taRightJustify;
  result := true
end;

{ TBulletButton }

constructor TBulletButton.Create(AOwner: TComponent);
begin
  inherited CreateFromBitMapRes(AOwner,'BULLET')
end;

function TBulletButton.Numbering(var NS: TNumberingStyle): boolean;
begin
  NS := nsBullet;
  result := true
end;



end.
